// Copyright (c) 2017, Tobias Mueller tm(at)tm3d.de
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are
// met:
//
//  * Redistributions of source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
//  * Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the following disclaimer in the
//    documentation and/or other materials provided with the
//    distribution.
//  * All advertising materials mentioning features or use of this
//    software must display the following acknowledgement: This product
//    includes software developed by tm3d.de and its contributors.
//  * Neither the name of tm3d.de nor the names of its contributors may
//    be used to endorse or promote products derived from this software
//    without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include "owDeviceConfig.h"
#include <math.h>

std::vector<std::string> owDeviceConfig::getFamilyInfo(uint8_t famcode) {
	std::vector<std::string> l;
	switch (famcode) {
			case 1:
				l.push_back("DS2401,DS2411");
				l.push_back("DS1990A");
				l.push_back("silicon serial number");
				break;
			case 2:
				l.push_back("DS1425");
				l.push_back("DS1991");
				l.push_back("multikey 1153bit secure");
				break;
			case 4:
				l.push_back("DS2404");
				l.push_back("DS1994");
				l.push_back("econoram time chip");
				break;
			case 5:
				l.push_back("DS2405");
				l.push_back("");
				l.push_back("push_backresable Switch");
				break;
			case 6:
				l.push_back(""); l.push_back("DS1993");
				l.push_back("4k memory ibutton");
				break;
			case 8:
				l.push_back(""); l.push_back("DS1992");
				l.push_back("1k memory ibutton");
				break;
			case 9:
				l.push_back("DS2502"); l.push_back("DS1982");
				l.push_back("1k add - only memory");
				break;
			case 0x0A: l.push_back(""); l.push_back("DS1995");
				l.push_back("16k memory ibutton"); break;
			case 0x0B: l.push_back("DS2505"); l.push_back("DS1985");
				l.push_back("16k add - only memory"); break;
			case 0x0C: l.push_back(""); l.push_back("DS1996");
				l.push_back("64k memory ibutton"); break;
			case 0x0F: l.push_back("DS2506"); l.push_back("1986");
				l.push_back("64k add - only  memory"); break;
			case 0x10:l.push_back("DS18S20"); l.push_back("DS1920");
				l.push_back("high precision digital thermometer"); break;
			case 0x12:l.push_back("DS2406,DS2407"); l.push_back("");
				l.push_back("dual addressable switch plus 1k memory"); break;
			case 0x14:l.push_back("DS2430A"); l.push_back("1971");
				l.push_back("256 eeprom"); break;
			case 0x1A:l.push_back(""); l.push_back("DS1963L");
				l.push_back("4k Monetary"); break;
			case 0x1B:l.push_back("DS2436"); l.push_back("");
				l.push_back("battery id / monitor chip"); break;
			case 0x1C:l.push_back("DS28E04-100"); l.push_back("");
				l.push_back("4k EEPROM with PIO	1C"); break;
			case 0x1D:l.push_back("DS2423"); l.push_back("");
				l.push_back("4k ram with counter"); break;
			case 0x1F:l.push_back("DS2409"); l.push_back("");
				l.push_back("microlan coupler"); break;
			case 0x20 : l.push_back("DS2450"); l.push_back("");
				l.push_back("quad a / d converter"); break;
			case 0x21: l.push_back(""); l.push_back("DS1921");
				l.push_back("Thermachron"); break;
			case 0x22: l.push_back("DS1822"); l.push_back("");
				l.push_back("Econo Digital Thermometer"); break;
			case 0x23: l.push_back("DS2433"); l.push_back("");
				l.push_back("4k eeprom"); break;
			case 0x24: l.push_back("DS2415"); l.push_back("");
				l.push_back("time chip"); break;
			case 0x26: l.push_back("DS2438"); l.push_back("");
				l.push_back("smart battery monitor"); break;
			case 0x27:l.push_back("DS2417"); l.push_back("");
				l.push_back("time chip with interrupt"); break;
			case 0x28:l.push_back("DS18B20"); l.push_back("");
				l.push_back("programmable resolution digital thermometer"); break;
			case 0x29:l.push_back("DS2408"); l.push_back("");
				l.push_back("8 - channel addressable switch"); break;
			case 0x2C:l.push_back("DS2890"); l.push_back("");
				l.push_back("digital potentiometer"); break;
			case 0x2D:l.push_back("DS2431"); l.push_back("");
				l.push_back("1k eeprom"); break;
			case 0x2E:l.push_back("DS2770"); l.push_back("");
				l.push_back("battery monitor and charge controller"); break;
			case 0x30:l.push_back("DS2760/61/62"); l.push_back("");
				l.push_back("high - precision li + battery monitor"); break;
			case 0x31:l.push_back("DS2720"); l.push_back("");
				l.push_back("efficient addressable single - cell rechargable lithium protection ic"); break;
			case 0x33: l.push_back("DS2432"); l.push_back("DS1961S");
				l.push_back("1k protected eeprom with SHA - 1"); break;
			case 0x36:l.push_back("DS2740"); l.push_back("");
				l.push_back("high precision coulomb counter"); break;
			case 0x37: l.push_back(""); l.push_back("DS1977"); 
				l.push_back("Password protected 32k eeprom"); break;
			case 0x3A:l.push_back("DS2413"); l.push_back("");
				l.push_back("dual channel addressable switch"); break;
			case 0x41:l.push_back("DS2422"); l.push_back("DS1922L/DS1922T/DS1923");
				l.push_back("Temperature Logger 8k mem"); break;
			case 0x51:l.push_back("DS2751"); l.push_back("");
				l.push_back("multichemistry battery fuel gauge"); break;
			case 0x81: l.push_back(""); l.push_back("DS1420");
				l.push_back("Serial ID Button"); break;
			case 0x84:l.push_back("DS2404S"); l.push_back("");
				l.push_back("dual port plus time"); break;
			case 0x89:l.push_back("DS2502-E48"); l.push_back("DS1982U");
				l.push_back("48 bit node address chip"); break;
			case 0x8B:l.push_back("DS2502/DS2505"); l.push_back("DS1985U");
				l.push_back("16k add - only"); break;
			case 0x8F:l.push_back("DS2506"); l.push_back("DS1986U");
				l.push_back("64k add - only uniqueware"); break;
			case 0xFF:l.push_back("LCD"); l.push_back("");
				l.push_back("(Swart)"); break;
			case 0xA3:l.push_back("ATTINY84A"); l.push_back("");
				l.push_back("1-Wire Flasher"); break;
			default:l.push_back("unknown"); l.push_back("unknown");
				l.push_back("Device not int list"); 
				break;
		}
			return l;
	}

std::vector<std::string> owDeviceConfig::getQuantityFromNumber(uint8_t code) {
	std::vector<std::string> l;
	switch (code) {
		case 1:
			l.push_back("temperature");
			l.push_back("Temperatur");
			l.push_back("°C");
			break;
		case 2:
			l.push_back("pressure");
			l.push_back("Druck");
			l.push_back("hPa");
			break;
		case 3:
			l.push_back("illuminance");
			l.push_back("Beleuchtungsstärke");
			l.push_back("lx");
			break;
		case 4:
			l.push_back("humidity");
			l.push_back("Luftfeuchte");
			l.push_back("%");
			break;
		case 5:
			l.push_back("constant");
			l.push_back("Konstante");
			l.push_back("");
			break;
		case 6:
			l.push_back("voltage");
			l.push_back("Spannung");
			l.push_back("V");
			break;
		case 7:
			l.push_back("current");
			l.push_back("Strom");
			l.push_back("mA");
			break;
		case 8:
			l.push_back("VOC");
			l.push_back("VOC");
			l.push_back("ppm");
		case 9:
			l.push_back("Counter");
			l.push_back("Zähler");
			l.push_back("");
			break;
		case 10:
			l.push_back("CO2");
			l.push_back("CO2");
			l.push_back("ppm");
			break;	
		case 11:
			l.push_back("resistance");
			l.push_back("Widerstand");
			l.push_back("kOhm");
			break;	
                case 12:
                        l.push_back("wind speed");
                        l.push_back("Windgeschwindigkeit");
                        l.push_back("m/s");
                        break;  
                case 13:
                        l.push_back("wind speed max");
                        l.push_back("Maximale Windgeschwindigkeit");
                        l.push_back("m/s");
                        break;  
                case 14:
                        l.push_back("wind direction");
                        l.push_back("Windrichtung");
                        l.push_back("°");
                        break;  
                case 15:
                        l.push_back("wind direction variance");
                        l.push_back("Varianz der Windrichtung");
                        l.push_back("°");
                        break;  
		case 16:
			l.push_back("illuminance red");
			l.push_back("Beleuchtungsstärke Rot");
			l.push_back("lx");
		case 17:
			l.push_back("illuminance green");
			l.push_back("Beleuchtungsstärke Grün");
			l.push_back("lx");
		case 18:
			l.push_back("illuminance blue");
			l.push_back("Beleuchtungsstärke Blau");
			l.push_back("lx");
		default:
			l.push_back("");
			l.push_back("");
			l.push_back("");
	
	}
	//Optimization necessary
	if (language==OWDC_LANGUAGE_ENGLISH) l.erase(l.begin()+1);
	if (language==OWDC_LANGUAGE_GERMAN) l.erase(l.begin()); 
	
	return l;
}

double owDeviceConfig::calculateValueFromNumber(int code, int vn, std::vector<int> V) {
	switch (code) {
			case 01: return V[vn] / 16.0; break;
			case 2:return V[vn]/1.6; break;
			case 3:return V[vn]*0.2 + 700; break;
			case 4:return exp(V[vn] / 160.0); break;
			case 5:return V[vn]*62.5 + 55000; break;
			case 6:return V[vn] / 256.0; break;
			case 7:return (((double)V[2] / (double)V[1] - 0.16) / 0.0062) / (1.0546 - 0.00216*V[0]/256.0); break;
			case 8:return V[vn] / 100.0; break;
			case 9:return V[vn] / 65535.0*5.1; break;
			case 10:return V[vn] / 65535.0*2.55; break;
			case 11:return V[vn] / 65535.0*1.1; break;
			case 12:return V[vn] / 10.0; break;
			case 13:return V[vn];  break;
			case 14:return (V[vn] - 32767.0) / 100.0;  break;
			case 15:return exp((V[vn]-32767.0)/1000.0);  break;
			case 16:return V[vn]/32.0;  break;
			case 17:return V[vn]*0.2441/1000.0; break; //Vsens DS2438
			case 18:return V[vn]/8.0;break;
			case 19:return V[vn]/500.0*14.0;break;  //Test
			case 20:return V[vn]*0.5+700;break;  
			case 21:return V[vn]+1280; break;  
			default:return 0;
		}
}

std::string owDeviceConfig::getSensorChip(uint8_t ch) {
	if (ch>4) return "";
	std::string s;
	switch(config.sens[ch]) {
		case 1: s="DS18B20";break;
		case 2: s="DS2438";break;	//Temperatur Sensor
		case 3: s="DS2438";break;	//Spannung
		case 4: s="DS2438";break;	//	Spannung / Strom
		case 5: s="DS2450";break;	//	Spannung
		case 6: s="Thermoelement";break;	//	Hochtemperatur Sensor
		case 7: s="SHT21";break;	//	Feuchte/Temperatur Sensor
		case 8: s="SHT25";break;	//	Feuchte/Temperatur Präzisionssensor
		case 9: s="DHT22";break;	//	Feuchte/Temperatur Sensor
		case 10: s="HIH9021";break;	//	Feuchte/Temperatur Präzisionssensor
		case 11: s="HDC1080";break;	//	Feuchte/Temperatur Sensor
		case 12: s="HIH4030";break;	//	Feuchte/Temperatur Sensor analog
		case 13: s="HIH5030";break;	//	Feuchte/Temperatur Sensor analog
		case 14: s="BMP280";break;	//	Luftdrucksensor
		case 15: s="MAX44009";break;	//	Umgebungslicht-Sensor
		case 16: s="CDM7160";break;	//	CO2-Sensor
		case 17: s="MAX1164/TGS8100";break;	// 	VOC-Sensor
		case 18: s="TGS8100 direkt";break;	//	VOC-Sensor
		case 19: s="DS2423";break;	//	Counter
		case 20: s="Intern ADC";break;
		case 21: s="SHT35";break;	//	Feuchte/Temperatur Bester
		case 22: s="SHT31";break;	//	Feuchte/Temperautr
		case 23: s="APDS-9960";break;	//	Feuchte/Temperautr
		case 24: s="BMP680";break;
		case 25: s="SGP30";break;
		case 26: s="SGPC3";break;
		default: return "";
	}
	return s;
}
